#include <konkret/konkret.h>
#include "LMI_Identity.h"

#include <libuser/entity.h>
#include <libuser/user.h>

#include "macros.h"
#include "globals.h"
#include "aux_lu.h"

static const CMPIBroker* _cb = NULL;

static void LMI_IdentityInitialize()
{
}

static CMPIStatus LMI_IdentityCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IdentityEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_IdentityEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    LMI_Identity li;
    struct lu_context *luc = NULL;
    struct lu_error *error = NULL;
    GPtrArray *accounts = NULL;
    struct lu_ent *lue = NULL;
    char *instanceid = NULL;
    unsigned int i = 0;
    const char *nameSpace = KNameSpace(cop);

    luc = lu_start(NULL, lu_user, NULL, NULL, lu_prompt_console_quiet, NULL, &error);
    if (!luc)
      {
        fprintf(stderr, "Error initializing: %s\n", lu_strerror(error));
        exit(1);
      }

    instanceid = malloc(256 * sizeof(char));

    /* USERS */
    accounts = lu_users_enumerate_full(luc, "*", &error);
    for (i = 0;  (accounts != NULL) && (i < accounts->len); i++)
      {
        lue = g_ptr_array_index(accounts, i);
        LMI_Identity_Init(&li, _cb, nameSpace);

        /* Need to convert long int UID to the string */
        snprintf(instanceid, 255, ORGID":UID:%ld",
          aux_lu_get_long(lue, LU_UIDNUMBER));
        LMI_Identity_Set_InstanceID(&li, instanceid);
        LMI_Identity_Set_ElementName(&li, aux_lu_get_str(lue, LU_GECOS));
        KReturnInstance(cr, li);
      }

    /* GOUPRS */
    accounts = lu_groups_enumerate_full(luc, "*", &error);
    for (i = 0;  (accounts != NULL) && (i < accounts->len); i++)
      {
        lue = g_ptr_array_index(accounts, i);
        LMI_Identity_Init(&li, _cb, nameSpace);

        /* Need to convert long int UID to the string */
        snprintf(instanceid, 255, ORGID":GID:%ld",
          aux_lu_get_long(lue, LU_GIDNUMBER));
        LMI_Identity_Set_InstanceID(&li, instanceid);
        LMI_Identity_Set_ElementName(&li, aux_lu_get_str(lue, LU_GROUPNAME));
        KReturnInstance(cr, li);
        lu_ent_free(lue);
      }

    free(instanceid);
    if (accounts)
      {
        g_ptr_array_free(accounts, TRUE);
      }
    lu_end(luc);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IdentityGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_IdentityCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IdentityModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IdentityDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    LMI_Identity identity;
    const char* instance_id = NULL;
    id_t id;
    char errmsg[256];
    struct lu_context *luc = NULL;
    struct lu_error *error = NULL;
    struct lu_ent *lue = NULL;

    LMI_Identity_InitFromObjectPath(&identity, _cb, cop);
    lue = lu_ent_new();
    instance_id = identity.InstanceID.chars;
    id = atol(rindex(instance_id, ':') + 1);

    luc = lu_start(NULL, 0, NULL, NULL, lu_prompt_console_quiet, NULL, &error);
    if (!luc)
      {
        fprintf(stderr, "Error initializing: %s\n", lu_strerror(error));
        exit(1);
      }

    if (strstr(instance_id, ":GID:"))
      { /* It's a group */
        if (!lu_group_lookup_id(luc, id, lue, &error))
          { /* User with that ID is not present */
            snprintf(errmsg, 256, "Non existing group id: %d\n", id);
            lu_ent_free(lue);
            lu_end(luc);
            CMReturnWithChars(_cb, CMPI_RC_ERR_NOT_FOUND, errmsg);
          }
        else
          {
            if (!lu_group_delete(luc, lue, &error))
              { /* user delete error */
                snprintf(errmsg, 256,
                  "Group with id %d could not be deleted: %s\n", id,
                  lu_strerror(error));
                lu_ent_free(lue);
                lu_end(luc);
                CMReturnWithChars(_cb, CMPI_RC_ERR_FAILED, errmsg);
              }
          }
      }
    else
      { /* It's an user */
        if (!lu_user_lookup_id(luc, id, lue, &error))
          { /* User with that ID is not present */
            snprintf(errmsg, 256, "Non existing user id: %d\n", id);
            lu_ent_free(lue);
            lu_end(luc);
            CMReturnWithChars(_cb, CMPI_RC_ERR_NOT_FOUND, errmsg);
          }
        else
          {
            if (!lu_user_delete(luc, lue, &error))
              { /* user delete error */
                snprintf(errmsg, 256,
                  "User with id %d could not be deleted: %s\n", id,
                  lu_strerror(error));
                lu_ent_free(lue);
                lu_end(luc);
                CMReturnWithChars(_cb, CMPI_RC_ERR_FAILED, errmsg);
              }
          }
      }
    lu_ent_free(lue);
    lu_end(luc);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IdentityExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_Identity,
    LMI_Identity,
    _cb,
    LMI_IdentityInitialize())

static CMPIStatus LMI_IdentityMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IdentityInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_Identity_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_Identity,
    LMI_Identity,
    _cb,
    LMI_IdentityInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_Identity",
    "LMI_Identity",
    "instance method")
